@@ -1,11 +1,12 @@
 /* snac - A simple, minimalistic ActivityPub instance */
-/* copyright (c) 2022 - 2024 grunfink et al. / MIT license */
+/* copyright (c) 2022 - 2025 grunfink et al. / MIT license */
 
 #include "xs.h"
 #include "xs_io.h"
 #include "xs_json.h"
 #include "xs_time.h"
 #include "xs_openssl.h"
+#include "xs_match.h"
 
 #include "snac.h"
 
@@ -14,7 +15,7 @@
 int usage(void)
 {
     printf("snac " VERSION " - A simple, minimalistic ActivityPub instance\n");
-    printf("Copyright (c) 2022 - 2024 grunfink et al. / MIT license\n");
+    printf("Copyright (c) 2022 - 2025 grunfink et al. / MIT license\n");
     printf("\n");
     printf("Commands:\n");
     printf("\n");
@@ -25,26 +26,38 @@ int usage(void)
     printf("httpd {basedir}                      Starts the HTTPD daemon\n");
     printf("purge {basedir}                      Purges old data\n");
     printf("state {basedir}                      Prints server state\n");
-    printf("webfinger {basedir} {actor}          Queries about an actor (@user@host or actor url)\n");
+    printf("webfinger {basedir} {account}        Queries about an account (@user@host or actor url)\n");
     printf("queue {basedir} {uid}                Processes a user queue\n");
     printf("follow {basedir} {uid} {actor}       Follows an actor\n");
     printf("unfollow {basedir} {uid} {actor}     Unfollows an actor\n");
     printf("request {basedir} {uid} {url}        Requests an object\n");
+    printf("insert {basedir} {uid} {url}         Requests an object and inserts it into the timeline\n");
     printf("actor {basedir} [{uid}] {url}        Requests an actor\n");
     printf("note {basedir} {uid} {text} [files...] Sends a note with optional attachments\n");
+    printf("note_unlisted {basedir} {uid} {text} [files...] Sends an unlisted note with optional attachments\n");
+    printf("note_mention {basedir} {uid} {text} [files...] Sends a note only to mentioned accounts\n");
     printf("boost|announce {basedir} {uid} {url} Boosts (announces) a post\n");
     printf("unboost {basedir} {uid} {url}        Unboosts a post\n");
     printf("resetpwd {basedir} {uid}             Resets the password of a user\n");
     printf("ping {basedir} {uid} {actor}         Pings an actor\n");
-    printf("webfinger_s {basedir} {uid} {actor}  Queries about an actor (@user@host or actor url)\n");
+    printf("webfinger_s {basedir} {uid} {account} Queries about an account (@user@host or actor url)\n");
     printf("pin {basedir} {uid} {msg_url}        Pins a message\n");
     printf("unpin {basedir} {uid} {msg_url}      Unpins a message\n");
+    printf("bookmark {basedir} {uid} {msg_url}   Bookmarks a message\n");
+    printf("unbookmark {basedir} {uid} {msg_url} Unbookmarks a message\n");
     printf("block {basedir} {instance_url}       Blocks a full instance\n");
     printf("unblock {basedir} {instance_url}     Unblocks a full instance\n");
     printf("limit {basedir} {uid} {actor}        Limits an actor (drops their announces)\n");
     printf("unlimit {basedir} {uid} {actor}      Unlimits an actor\n");
+    printf("unmute {basedir} {uid} {actor}       Unmutes a previously muted actor\n");
     printf("verify_links {basedir} {uid}         Verifies a user's links (in the metadata)\n");
     printf("search {basedir} {uid} {regex}       Searches posts by content\n");
+    printf("export_csv {basedir} {uid}           Exports data as CSV files\n");
+    printf("alias {basedir} {uid} {account}      Sets account (@user@host or actor url) as an alias\n");
+    printf("migrate {basedir} {uid}              Migrates to the account defined as the alias\n");
+    printf("import_csv {basedir} {uid}           Imports data from CSV files\n");
+    printf("import_list {basedir} {uid} {file}   Imports a Mastodon CSV list file\n");
+    printf("import_block_list {basedir} {uid} {file} Imports a Mastodon CSV block list file\n");
 
     return 1;
 }
@@ -84,19 +97,6 @@ int main(int argc, char *argv[])
         return snac_init(basedir);
     }
 
-    if (strcmp(cmd, "upgrade") == 0) { /** **/
-        int ret;
-
-        /* upgrade */
-        if ((basedir = GET_ARGV()) == NULL)
-            return usage();
-
-        if ((ret = srv_open(basedir, 1)) == 1)
-            srv_log(xs_dup("OK"));
-
-        return ret;
-    }
-
     if (strcmp(cmd, "markdown") == 0) { /** **/
         /* undocumented, for testing only */
         xs *c = xs_readall(stdin);
@@ -106,8 +106,20 @@ int main(int argc, char *argv[])
         return 0;
     }
 
-    if ((basedir = GET_ARGV()) == NULL)
-        return usage();
+    if ((basedir = getenv("SNAC_BASEDIR")) == NULL) {
+        if ((basedir = GET_ARGV()) == NULL)
+            return usage();
+    }
+
+    if (strcmp(cmd, "upgrade") == 0) { /** **/
+        int ret;
+
+        /* upgrade */
+        if ((ret = srv_open(basedir, 1)) == 1)
+            srv_log(xs_dup("OK"));
+
+        return ret;
+    }
 
     if (!srv_open(basedir, 0)) {
         srv_log(xs_fmt("error opening data storage at %s", basedir));
@@ -264,9 +276,53 @@ int main(int argc, char *argv[])
         return 0;
     }
 
+    if (strcmp(cmd, "export_csv") == 0) { /** **/
+        export_csv(&snac);
+        return 0;
+    }
+
+    if (strcmp(cmd, "import_csv") == 0) { /** **/
+        import_csv(&snac);
+        return 0;
+    }
+
+    if (strcmp(cmd, "migrate") == 0) { /** **/
+        return migrate_account(&snac);
+    }
+
     if ((url = GET_ARGV()) == NULL)
         return usage();
 
+    if (strcmp(cmd, "alias") == 0) { /** **/
+        xs *actor = NULL;
+        xs *uid = NULL;
+        int status = HTTP_STATUS_OK;
+
+        if (*url == '\0')
+            actor = xs_dup("");
+        else
+            status = webfinger_request(url, &actor, &uid);
+
+        if (valid_status(status)) {
+            if (strcmp(actor, snac.actor) == 0) {
+                snac_log(&snac, xs_fmt("You can't be your own alias"));
+                return 1;
+            }
+            else {
+                snac.config = xs_dict_set(snac.config, "alias", actor);
+                snac.config = xs_dict_set(snac.config, "alias_raw", url);
+
+                user_persist(&snac, 1);
+            }
+        }
+        else {
+            snac_log(&snac, xs_fmt("Webfinger error for %s %d", url, status));
+            return 1;
+        }
+
+        return 0;
+    }
+
     if (strcmp(cmd, "webfinger_s") == 0) { /** **/
         xs *actor = NULL;
         xs *uid = NULL;
@@ -297,6 +353,22 @@ int main(int argc, char *argv[])
         return 0;
     }
 
+
+    if (strcmp(cmd, "assist") == 0) { /** **/
+        /* undocumented: experimental (do not use) */
+        xs *msg = msg_admiration(&snac, url, "Accept");
+
+        if (msg != NULL) {
+            enqueue_message(&snac, msg);
+
+            if (dbglevel) {
+                xs_json_dump(msg, 4, stdout);
+            }
+        }
+
+        return 0;
+    }
+
     if (strcmp(cmd, "unboost") == 0) { /** **/
         xs *msg = msg_repulsion(&snac, url, "Announce");
 
@@ -375,6 +447,18 @@ int main(int argc, char *argv[])
         return 0;
     }
 
+    if (strcmp(cmd, "unmute") == 0) { /** **/
+        if (is_muted(&snac, url)) {
+            unmute(&snac, url);
+
+            printf("%s unmuted\n", url);
+        }
+        else
+            printf("%s actor is not muted\n", url);
+
+        return 0;
+    }
+
     if (strcmp(cmd, "search") == 0) { /** **/
         int to;
 
@@ -442,6 +526,26 @@ int main(int argc, char *argv[])
         return 0;
     }
 
+    if (strcmp(cmd, "bookmark") == 0) { /** **/
+        int ret = bookmark(&snac, url);
+        if (ret < 0) {
+            fprintf(stderr, "error bookmarking %s %d\n", url, ret);
+            return 1;
+        }
+
+        return 0;
+    }
+
+    if (strcmp(cmd, "unbookmark") == 0) { /** **/
+        int ret = unbookmark(&snac, url);
+        if (ret < 0) {
+            fprintf(stderr, "error unbookmarking %s %d\n", url, ret);
+            return 1;
+        }
+
+        return 0;
+    }
+
     if (strcmp(cmd, "question") == 0) { /** **/
         int end_secs = 5 * 60;
         xs *opts = xs_split(url, ";");
@@ -476,6 +580,27 @@ int main(int argc, char *argv[])
         return 0;
     }
 
+    if (strcmp(cmd, "insert") == 0) { /** **/
+        int status;
+        xs *data = NULL;
+
+        status = activitypub_request(&snac, url, &data);
+
+        printf("status: %d\n", status);
+
+        if (data != NULL) {
+            xs_json_dump(data, 4, stdout);
+            enqueue_actor_refresh(&snac, xs_dict_get(data, "attributedTo"), 0);
+
+            if (!timeline_here(&snac, url))
+                timeline_add(&snac, url, data);
+            else
+                printf("Post %s already here\n", url);
+        }
+
+        return 0;
+    }
+
     if (strcmp(cmd, "request2") == 0) { /** **/
         enqueue_object_request(&snac, url, 2);
 
@@ -497,7 +622,21 @@ int main(int argc, char *argv[])
         return 0;
     }
 
-    if (strcmp(cmd, "note") == 0) { /** **/
+    if (strcmp(cmd, "import_list") == 0) { /** **/
+        import_list_csv(&snac, url);
+
+        return 0;
+    }
+
+    if (strcmp(cmd, "import_block_list") == 0) { /** **/
+        import_blocked_accounts_csv(&snac, url);
+
+        return 0;
+    }
+
+    if (strcmp(cmd, "note") == 0 ||             /** **/
+        strcmp(cmd, "note_unlisted") == 0 ||    /** **/
+        strcmp(cmd, "note_mention") == 0) {     /** **/
         xs *content = NULL;
         xs *msg = NULL;
         xs *c_msg = NULL;
@@ -561,7 +700,14 @@ int main(int argc, char *argv[])
         else
             content = xs_dup(url);
 
-        msg = msg_note(&snac, content, NULL, NULL, attl, 0);
+        int scope = 0;
+        if (strcmp(cmd, "note_mention") == 0)
+            scope = 1;
+        else
+        if (strcmp(cmd, "note_unlisted") == 0)
+            scope = 2;
+
+        msg = msg_note(&snac, content, NULL, NULL, attl, scope, getenv("LANG"));
 
         c_msg = msg_create(&snac, msg);
 
